; Speckey - AT Keyboard to ZX-Spectrum interface
; 
; Target device is ATtiny45
;
; v0.0
; To do
; =====
;
; 1.  Timeout checking on xfer to the keyboard
; 2.  Error handling on keyboard problems
;	  -- reset and restart


; PORT  Port B0 = KEY_CLOCK
; PORT  Port B1 = KEY_DATA


; PORT B4 = M_RESET
; PORT B3 = M_DATA
; PORT B2 = M_CLK

.include "tn45def.inc"


.equ	MEM_RESET 	= PORTB4
.equ	MEM_DATA  	= PORTB3
.equ	MEM_CLK   	= PORTB2

.equ	KEY_CLOCK	= PORTB0
.equ	KEY_DATA	= PORTB1

.equ 	KEY_BUFFER_START	= 0x60
.equ 	KEY_BUFFER_END		= 0xA0

.equ  	SPEC_KEY_BUFFER		= 0xC0


.def    rtemp		= r16
.def 	rtemp2		= r17
.def 	w_reg		= r18


.def  	rd_bit_cntr	= r19
.def 	rd_byte		= r20


.def	mem_rd_reg	= r0
.def 	rd_done		= r1
.def	keys_rdy	= r2
.def 	nxt_code	= r3

.def    edg_flag	= r4


.def 	key_off		= r5
.def	last_key	= r6





.equ 	the_baud = 25

; ----------------------------------------------

.MACRO  DEBUGGING
	ret
	nop
.ENDMACRO                       

; ----------------------------------------------




 rjmp reset ; Reset Handler
 rjmp int_0 ; External Interrupt0 Handler
 rjmp PC_int_0 ; External Interrupt1 Handler

; rjmp TIM1_CAPT ; Timer1 Capture Handler
; rjmp TIM1_COMPA ; Timer1 CompareA Handler
; rjmp TIM1_OVF ; Timer1 Overflow Handler
; rjmp TIM0_OVF ; Timer0 Overflow Handler
; rjmp USART0_RXC ; USART0 RX Complete Handler
; rjmp USART0_DRE ; USART0,UDR Empty Handler
; rjmp USART0_TXC ; USART0 TX Complete Handler
; rjmp ANA_COMP ; Analog Comparator Handler
; rjmp PCINT ; Pin Change Interrupt
; rjmp TIMER1_COMPB ; Timer1 Compare B Handler
; rjmp TIMER0_COMPA ; Timer0 Compare A Handler
; rjmp TIMER0_COMPB ; Timer0 Compare B Handler
; rjmp USI_START ; USI Start Handler
; rjmp USI_OVERFLOW ; USI Overflow Handler
; rjmp EE_READY ; EEPROM Ready Handler
; rjmp WDT_OVERFLOW ; Watchdog Overflow Handler




; --------------------------------------------------------------------------
; (*********************************************************************** )
; (*********************************************************************** )

reset:

; (*********************************************************************** )
; (*********************************************************************** )
; --------------------------------------------------------------------------

 
	cli




	; normal proper reset stuff


	;ldi rtemp, 0x51
	;out osccal, rtemp

	
	ldi rtemp,LOW(RAMEND)
	out spl,rtemp

	; clear the working registers
	; memory etc.

	rcall KBD_reset_vars
	






	; set port B(0,1) to INPUT  ( 2,3,4 )  to output
	
	clr rtemp
	out portb, rtemp

	ldi rtemp, 0b00011100
	out ddrb, rtemp

	sbi portb, KEY_CLOCK
	sbi portb, KEY_DATA


	; signal reset memory counter to the CPLD
	cbi PORTB, MEM_RESET 


	; clear all the CPLD memory locations
	rcall send_keystatus 



	 
	; set pc_int_0 active
	; enable interrupts
	 ;
	; set key clock to interrupt ( PB2 )

	ldi rtemp, ( 1 << PCINT0 )
	out PCMSK, rtemp 


	; clear GIFR make sure no IRQ's pending

	in  rtemp, GIFR
	out GIFR, rtemp


	; enable external interrupt 
		
	in  rtemp, GIMSK
	ori rtemp, ( 1 << PCIE )
	out GIMSK, rtemp


	; set the interrupt mode 
	; falling edge on INT0 (PB2) generates interrupt

	in rtemp, MCUCR
	ori rtemp, ( 1<<ISC01 )|(0<<ISC00)
	out MCUCR, rtemp



	; the timeout timer -- set to about 100uSec
	; if nothing has signalled in that length of time 
	; then the bit clock has gotten out of sync.

;	clr rtemp
;	out TCCR0A, rtemp
;
;	ldi rtemp, 0
;	out TCNT0, rtemp
;
;
;	; need to wait for keyboard to reset
;	; so delay around 500ms before proceeding
;
;
;	ldi rtemp, (1 << CS02) | (0 << CS01) | (1 << CS00)
;	out TCCR0B, rtemp
;
;	ldi rtemp2, 15 
;
;sud_timeout:
;	in rtemp, TIFR
;	sbrs rtemp, TOV0
;	rjmp sud_timeout
;
;	; clear the bit counter and reset the timer (!!)
;
;	ldi rtemp, ( 1 << TOV0 )
;	out TIFR, rtemp
;
;	dec rtemp2
;	brne sud_timeout
;	

	; stop the counter & set base parameters

	ldi rtemp, (0 << CS02) | (0 << CS01) | (0 << CS00)
	out TCCR0B, rtemp

	clr rtemp
	out TCCR0A, rtemp

	ldi rtemp, 255 - 100 ; 100uSec per tick
	out TCNT0, rtemp




systems_are_go:

	sei  	; interrupts are go !!


	; ------------------------------------------
	; ----------- start the key processing !!!
	; ------------------------------------------


WAIT_FOR_KEYS:
	
	; check the timer and see if it's overflowed
	; if so.. CLEAR the 'rd_bit_cntr ' reg

	in rtemp, TIFR
	sbrs rtemp, TOV0
	rjmp not_timed_out

;	; clear the bit counter and reset the timer (!!)

	ldi rtemp, ( 1 << TOV0 )
	out TIFR, rtemp
	ldi rtemp, (0 << CS02) | (0 << CS01) | (0 << CS00)
	out TCCR0B, rtemp

	clr rd_bit_cntr


; debugging only
;
;	ldi rtemp, (0 << CS02) | (0 << CS01) | (0 << CS00)
;	out TCCR0B, rtemp
;	ldi rtemp, 255 - 100 ; 100uSec per tick
;	out TCNT0, rtemp

	


not_timed_out:

	
	tst keys_rdy
	breq WAIT_FOR_KEYS


	; there are keys to process !!!!


	; get the next key from the buffer
	ld rtemp, Y+
	dec keys_rdy


	
	cpi Yl, KEY_BUFFER_END
	brne not_at_last_Y 
	ldi Yl, KEY_BUFFER_START

not_at_last_Y:
	

testit:

	; check for the init key code
	; ie. plugged in, just reset everything (!!)

	cpi rtemp, 0xAA
	brne not_self_test

	rjmp KBD_init_all

not_self_test:


	; rtemp = the last key pressed


	; it's a key off so wait to find which key
	cpi rtemp, 0xF0
	brne not_key_off

	dec key_off
	rjmp WAIT_FOR_KEYS

not_key_off:

	andi rtemp, 127
	add rtemp, rtemp


	ldi Zh, high( 2 * speccy_LUT )
	ldi Zl, low( 2 * speccy_LUT )
	add zl, rtemp
	brcc no_inc_z
	inc Zh

no_inc_z:		

	LPM	 rtemp2, Z+		
	LPM  nxt_code, Z+


sec_key_process:

	mov  rtemp, rtemp2
	andi rtemp, 0x7     ; the offset

	lsr rtemp2
	lsr rtemp2
	lsr rtemp2 		; shift down the upper bits


	; the speccy keybit storage area

	; $C0 to $C8 - the 8 rows of the keyboard

	ldi Zh, 0
	ldi Zl, SPEC_KEY_BUFFER
	add zl, rtemp
	ld rtemp, Z
	

	tst key_off
	breq ok_not_off

	; if the key is OFF then mask out the bits
	; that should be cleared

	com rtemp2
	and rtemp2, rtemp

	rjmp 	save_rslt


ok_not_off:
	; set the bits that should be on 
	or rtemp2, rtemp


save_rslt:

	st Z, rtemp2


	; see if there's a second key to process !
	tst nxt_code
	breq no_extra_keys


	mov rtemp2, nxt_code
	clr nxt_code
	rjmp sec_key_process



	; --- end of processing the keys

no_extra_keys:



show_keys:
end_outp:




	tst key_off
	breq dclr
	clr last_key ; clear the last key in a key-off situation	
		
dclr:

	; transmit the statuses

	rcall send_keystatus


	clr key_off


	rcall LEDs
	; send the scan codes for the keys



	rjmp WAIT_FOR_KEYS

; --------------------------------------------------------------------------
; send the kb status to the PLD
; --------------------------------------------------------------------------

send_keystatus:


	; address A14

	
	; clear portB4 - end the counter reset
	CBI PORTB, MEM_RESET

	nop
	nop
	nop
	nop
	nop
	nop ; allow it to settle

	; address A8 keybits
	lds rtemp2, $C0
	rcall output_the_keys

	; address A9  keybits
	lds rtemp2, $C1
	rcall output_the_keys

	; address A10 keybits
	lds rtemp2, $C2
	rcall output_the_keys

	; address A11 keybits
	lds rtemp2, $C3
	rcall output_the_keys

	; address A12 keybits
	lds rtemp2, $C4
	rcall output_the_keys

	; address A13 keybits
	lds rtemp2, $C5
	rcall output_the_keys

	; address A14 keybits
	lds rtemp2, $C6
	rcall output_the_keys

	; address A15 keybits
	lds rtemp2, $C7
	rcall output_the_keys


	; set MEM_RESET - reset the key bit counter
	nop
	nop
	nop
	nop

	SBI PORTB, MEM_RESET


	ret


; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
output_the_keys:
; --------------------------------------------------------------------------
; --------------------------------------------------------------------------



	; 5 bits for each of the spectrum key rows

	; data is placed on the mem_data line
	; and then on the rising edge of mem_clk
	; the CPLD clocks in the bit
	; on the falling edge of mem_clk the
	; CPLD increments the address for the next
	; write

	; '1' = active the CPLD inverts the bits when
	; is passes the key states to the spectrum


	ldi rtemp, 5 


opk_lop1:

	; clock to '0'
	CBI PORTB, MEM_CLK
	nop
	nop
	nop



	CBI PORTB, MEM_DATA
	
	lsr rtemp2
	brcc opk_not1
	SBI PORTB, MEM_DATA

opk_not1:
	nop
	nop
	nop
	nop

	; clock bit into the CPLD
	SBI PORTB, MEM_CLK

	nop
	nop
	nop
	nop

	; clock the bit low change next byte
	CBI PORTB, MEM_CLK
	


	dec rtemp
	brne opk_lop1




	ret


; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
KBD_reset_vars:
; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
 
	; reset the registars and memory used
	

	cli

	clr rtemp
	sts $C0, rtemp
	sts $C1, rtemp
	sts $C2, rtemp
	sts $C3, rtemp
	sts $C4, rtemp
	sts $C5, rtemp
	sts $C6, rtemp
	sts $C7, rtemp

	; clear the working registers

	clr rd_bit_cntr
	clr rd_byte
	clr rd_done
	clr keys_rdy
	clr edg_flag
	clr key_off
	clr last_key



	; the keyboard buffer save pointer
	; and read pointers


	ldi Xh, HIGH(KEY_BUFFER_START)
	ldi XL, LOW(KEY_BUFFER_START)

	ldi Yh, HIGH( KEY_BUFFER_START )
	ldi YL, LOW(KEY_BUFFER_START )

	ret


; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
KBD_init_all:
; --------------------------------------------------------------------------
; --------------------------------------------------------------------------



	rcall KBD_reset_vars
	rcall keyboard_config

	rjmp systems_are_go


; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
host_to_keyboard:
; --------------------------------------------------------------------------
; --------------------------------------------------------------------------


	mov rtemp2, rtemp
	rcall key_xfer

htk_ack1:

	tst keys_rdy
	breq htk_ack1

htk_wt_empty:

	rcall get_new_key


	tst keys_rdy
	brne htk_wt_empty
	
	ret



; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
keyboard_config:


	; -- turn on the middle LED


	ldi rtemp, $ED
	rcall host_to_keyboard
	nop
	nop
	nop
	nop
	nop

	ldi rtemp, $07
	rcall host_to_keyboard

	nop
	nop
	nop
	nop
	nop


	; Set the keyboard Scan Code set

	nop
	nop
	nop
	nop
	nop


	ldi rtemp, $F0
	rcall host_to_keyboard

	nop
	nop
	nop
	nop
	nop



	; set keyboard to scan code 3 which means NO auto-repeat
	ldi rtemp, $03	
	rcall host_to_keyboard


	nop
	nop
	nop
	nop
	nop




	; set all keys to Make/Break
	ldi rtemp, $F8
	rcall host_to_keyboard
	

	; check to see if the response is ok
	cpi rtemp, $FA
	brne bad_keyinit



	; -- set LED status

	ldi rtemp, $ED
	rcall host_to_keyboard

	ldi rtemp, $01
	rcall host_to_keyboard


	ret


bad_keyinit:

	; -- set LED status for error during init
	;  need to add better error handling here
	;  instead of just saying an error happened

	ldi rtemp, $ED
	rcall host_to_keyboard

	ldi rtemp, $02
	rcall host_to_keyboard




	ret



; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
key_xfer:
; --------------------------------------------------------------------------
; --------------------------------------------------------------------------


	cli
;	rjmp xfer_rdy



	; DISABLE INTERRUPTS FOR THE MINUTE

	clr  rtemp
	out  GIMSK, rtemp



	; clock must be high
	; in which case just wait for it to go high
	
	rcall keyclk_wait_hi


	; set PORTB 1 to OUTPUT LOW
	; data port is now LOW

	cbi PORTB, KEY_DATA	
	sbi DDRB, DDB1


	; loop thru' 8 bits of data

	ldi rtemp, 8 
	clr w_reg


	; wait for clock to go LOW->HIGH - the start bit (!)
	rcall keyclk_wait_lo



	; ============= SEND DATA TO THE KEYBOARD

ks_bit_lop:

	
	sbrs rtemp2, 0
	rjmp its_a_zero

	sbi PORTB, KEY_DATA
	inc w_reg
	rjmp ctin


 its_a_zero:
	cbi PORTB, KEY_DATA

ctin:

	lsr rtemp2
	
	; wait for clock to go LOW->HIGH



	rcall keyclk_wait_hi
	rcall keyclk_wait_lo



	nop
	nop
	nop


	dec rtemp
	brne ks_bit_lop




	;  wait for the last bit of the databus to go high !

	; ------------------------ 
	; bits 9,10,11
	; PARITY



	nop
	

	; send the parity

	sbrc w_reg, 0
	cbi PORTB, KEY_DATA

	sbrs w_reg, 0
	sbi PORTB, KEY_DATA

	

	rcall keyclk_wait_hi
	rcall keyclk_wait_lo

	; end of parity send

;	cbi PORTBD, PORTD5



	; ** WAIT FOR THE STOP BIT



	cbi DDRB, DDB1
	cbi PORTB, KEY_DATA


	rcall keyclk_wait_lo
	rcall keyclk_wait_hi



	; ** WAIT FOR THE ACK TO END

	rcall keyclk_wait_lo
	rcall keyclk_wait_hi




xfer_rdy:

	; reset the keyboard buffer pointers

	CLR rd_bit_cntr
	clr keys_rdy
	ldi XL, KEY_BUFFER_START
	ldi YL, KEY_BUFFER_START

	in  rtemp, GIFR
	out GIFR, rtemp


	in  rtemp, GIMSK
	ori rtemp, ( 1 << PCIF )
	out GIMSK, rtemp

	; re-enable the interrupts !
	sei

	ret


keyclk_wait_lo:

	; ------------------------------------------
kclk_wait_clk_lo:	
	sbic PINB, KEY_CLOCK
	rjmp kclk_wait_clk_lo

	ret

	; ------------------------------------------
keyclk_wait_hi:

kclk_wait_clk_hi:	
	sbis PINB, KEY_CLOCK
	rjmp kclk_wait_clk_hi

	ret


; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
get_new_key:
; --------------------------------------------------------------------------
; --------------------------------------------------------------------------


	ld rtemp, Y+
	dec keys_rdy

	cpi Yl, KEY_BUFFER_END
	brne not_at_last_Ygnk 
	ldi YL, KEY_BUFFER_START

not_at_last_Ygnk:

	ret


LEDs:
	; check for the shift keys

	clr rtemp2

	lds rtemp, $c7
	sbrs rtemp, 1
	rjmp LED_no_right

	inc rtemp2

LED_no_right:


	lds rtemp, $c0
	sbrs rtemp, 0
	rjmp LED_no_left
	
	inc rtemp2
	inc rtemp2

LED_no_left:	

	push rtemp2
	ldi rtemp, $ED
	rcall host_to_keyboard
	pop rtemp2

	lsl rtemp2
	inc rtemp2
	
	mov rtemp, rtemp2
	rcall host_to_keyboard

	ret


; --------------------------------------------------------------------------
; (*********************************************************************** )
; --------------------------------------------------------------------------
PC_int_0:
; --------------------------------------------------------------------------
; (*********************************************************************** )
; --------------------------------------------------------------------------

	; interrupt processing
	
	push rtemp
	in rtemp, SREG
	push rtemp


	; check if the pin is low, pin change i
	; low process the interrupt

	SBIC PINB, KEY_CLOCK
	RJMP int_0_endit

	
	; reset the timer
	ldi rtemp, 255 - 100 ; 100uSec per tick
	out TCNT0, rtemp


	inc rd_bit_cntr
	

	; get the current bit
	cpi rd_bit_cntr, 1
	breq skip_first_bit



	; bit 10 is parity.. umm.. ignore..(eek!)	
	cpi rd_bit_cntr, 10
	breq skip_par_bit


	; last bit is stop this we can ignore
	cpi rd_bit_cntr, 11
	breq skip_last_bit



	; *** shift in the bits from the port B(1)

	lsr rd_byte	
	sbic PINB, 1
	ori rd_byte, 128		
	rjmp int_0_endit		
	

	; ------------------------------------------
skip_first_bit:


	; start the timer clk/8 ( ie. 1uSec per tick )

	ldi rtemp, (0 << CS02) | (1 << CS01) | (0 << CS00)
	out TCCR0B, rtemp
	rjmp int_0_endit		



	; ------------------------------------------
skip_par_bit:
	rjmp int_0_endit		



	; ------------------------------------------
skip_last_bit:

	; if it's the same key as last time
	; just ignore it, we don't need auto-repeat etc.

	;	cp last_key, rd_byte
	;	breq skp_full


	mov rtemp, keys_rdy
	cpi rtemp, 0x80
	breq skp_full	

	inc keys_rdy
	ST X+,  rd_byte

	; check to see if at the last buffer address	
	cpi Xl, KEY_BUFFER_END
	brne not_at_last_addr
	
	ldi Xl, KEY_BUFFER_START
not_at_last_addr:


	
skp_full:

	; disable the timer as all bits received
	 

	ldi rtemp, (0 << CS02) | (0 << CS01) | (0 << CS00)
	out TCCR0B, rtemp


	; clear the bit counter
	clr rd_bit_cntr
	rjmp int_0_endit		



	; wrap up the Interrupt
int_0_endit:


	pop rtemp
	out SREG, rtemp
	pop rtemp

	reti



; --------------------------------------------------------------------------
int_0:
; --------------------------------------------------------------------------
	; placeholder for external interrupt INT0
	; not used at this time
	reti


	.org $380




; --------------------------------------------------------------------------
; --------------------------------------------------------------------------
speccy_LUT:
; --------------------------------------------------------------------------
; --------------------------------------------------------------------------

; form of the table is
; 
; value to set A 
; value to set B
;
; depending on the key pressed it can set 5 bits on upto TWO 
; spectrum rows ( A or B )
; 
; the bit format of the 'keybits':
; 
; Bit#
;   7  speccy key bit 4 
;   6  speccy key bit 3 
;   5  speccy key bit 2 
;   4  speccy key bit 1 
;   3  speccy key bit 0 
;
; 210  speccy key row ( 0 to 7 ) 
;   
; --
; 8 rows * 5 bits = 40 keys


; keybits A       keybits B    AT key number
; 
.db 0            ,0           ;0x00
.db 0            ,0           ;0x01
.db 0            ,0           ;0x02
.db 0            ,0           ;0x03
.db 0            ,0           ;0x04
.db 0            ,0           ;0x05
.db 0            ,0           ;0x06
.db 0            ,0           ;0x07
.db 0b00001000   ,0b00001111  ;0x08  ESC - caps shift - SPACE
.db 0            ,0           ;0x09
.db 0            ,0           ;0x0a
.db 0            ,0           ;0x0b
.db 0            ,0           ;0x0c
.db 0            ,0           ;0x0d  TAB
.db 0            ,0           ;0x0e  '~'
.db 0            ,0           ;0x0f
.db 0            ,0           ;0x10
.db 0            ,0           ;0x11
.db 0b00001000   ,0           ;0x12  left shift - CAPS SHIFT
.db 0            ,0           ;0x13
.db 0            ,0           ;0x14
.db 0b00001010   ,0           ;0x15  'q'
.db 0b00001011   ,0           ;0x16  '1' ;
.db 0            ,0           ;0x17
.db 0            ,0           ;0x18
.db 0            ,0           ;0x19
.db 0b00010000   ,0           ;0x1a  'z'
.db 0b00010001   ,0           ;0x1b  's'
.db 0b00001001   ,0           ;0x1c  'a'
.db 0b00010010   ,0           ;0x1d  'w'
.db 0b00010011   ,0           ;0x1e  '2'
.db 0            ,0           ;0x1f
.db 0            ,0           ;0x20
.db 0b01000000   ,0           ;0x21  'c'
.db 0b00100000   ,0           ;0x22  'x'
.db 0b00100001   ,0           ;0x23  'd'
.db 0b00100010   ,0           ;0x24  'e'
.db 0b01000011   ,0           ;0x25  '4'
.db 0b00100011   ,0           ;0x26  '3'
.db 0            ,0           ;0x27
.db 0            ,0           ;0x28
.db 0b00001111   ,0           ;0x29  Space
.db 0b10000000   ,0           ;0x2a  'v'
.db 0b01000001   ,0           ;0x2b  'f'
.db 0b10000010   ,0           ;0x2c  't'
.db 0b01000010   ,0           ;0x2d  'r'
.db 0b10000011   ,0           ;0x2e  '5'
.db 0            ,0           ;0x2f
.db 0            ,0           ;0x30
.db 0b01000111   ,0           ;0x31  'n'
.db 0b10000111   ,0           ;0x32  'b'
.db 0b10000110   ,0           ;0x33  'h'
.db 0b10000001   ,0           ;0x34  'g'
.db 0b10000101   ,0           ;0x35  'y'
.db 0b10000100   ,0           ;0x36  '6'
.db 0            ,0           ;0x37
.db 0            ,0           ;0x38
.db 0            ,0           ;0x39  ','
.db 0b00100111   ,0           ;0x3a  'm'
.db 0b01000110   ,0           ;0x3b  'j' 
.db 0b01000101   ,0           ;0x3c  'u'
.db 0b01000100   ,0           ;0x3d  '7'
.db 0b00100100   ,0           ;0x3e  '8'
.db 0            ,0           ;0x3f
.db 0            ,0           ;0x40
.db 0b00010111   ,0b01000111  ;0x41  ',' -- (symbol shift) 'N'
.db 0b00100110   ,0           ;0x42  'k'
.db 0b00100101   ,0           ;0x43  'i'
.db 0b00010101   ,0           ;0x44  'o'
.db 0b00001100   ,0           ;0x45  '0'
.db 0b00010100   ,0           ;0x46  '9'
.db 0            ,0           ;0x47
.db 0            ,0           ;0x48
.db 0b00010111   ,0b00100111  ;0x49  '.' -- (symbol shift) 'M'
.db 0            ,0           ;0x4a  '-'
.db 0b00010110   ,0           ;0x4b  'L'
.db 0b00010111   ,0b00010101  ;0x4c  ';' -- (symbol shift) 'O'
.db 0b00001101   ,0           ;0x4d  'p'
.db 0            ,0           ;0x4e  '-'
.db 0            ,0           ;0x4f
.db 0            ,0           ;0x50
.db 0            ,0           ;0x51
.db 0b00010111   ,0b00001101  ;0x52  '"' -- (symbol shift) 'P'
.db 0            ,0           ;0x53
.db 0            ,0           ;0x54  '{'
.db 0            ,0           ;0x55  
.db 0            ,0           ;0x56
.db 0            ,0           ;0x57
.db 0            ,0           ;0x58
.db 0b00010111   ,0           ;0x59  right shift ( symbol shift )
.db 0b00001110   ,0           ;0x5a  enter
.db 0            ,0           ;0x5b  '}'
.db 0            ,0           ;0x5c
.db 0            ,0           ;0x5d  '\'
.db 0            ,0           ;0x5e
.db 0            ,0           ;0x5f
.db 0b00001000   ,0b10000100  ;0x60  cursor down
.db 0b00001000   ,0b10000011  ;0x61  cursor left
.db 0            ,0           ;0x62
.db 0b00001000   ,0b01000100  ;0x63  cursor up
.db 0b00001000   ,0b00001100  ;0x64  'del' key        -- SHIFT + 0
.db 0            ,0           ;0x65
.db 0b00001000   ,0b00001100  ;0x66  backspace        -- SHIFT + 0
.db 0            ,0           ;0x67
.db 0            ,0           ;0x68
.db 0            ,0           ;0x69  '1'- keypad
.db 0b00001000   ,0b00100100  ;0x6a  ; cursor right
.db 0b00001000   ,0b10000011  ;0x6b  '4'- keypad  -- shift '5'
.db 0            ,0           ;0x6c  '7'- keypad  
.db 0            ,0           ;0x6d
.db 0            ,0           ;0x6e
.db 0            ,0           ;0x6f
.db 0            ,0           ;0x70  '0'- keypad
.db 0            ,0           ;0x71  '.'- keypad
.db 0b00001000   ,0b10000100  ;0x72  '2'- keypad  -- shift '6'
.db 0            ,0           ;0x73  '5'- keypad
.db 0b00001000   ,0b00100100  ;0x74  '6'- keypad  -- shift '8'
.db 0b00001000   ,0b01000100  ;0x75  '8'- keypad  -- shift '7'
.db 0            ,0           ;0x76
.db 0            ,0           ;0x77
.db 0            ,0           ;0x78
.db 0            ,0           ;0x79  '+'- keypad
.db 0            ,0           ;0x7a  '3'- keypad
.db 0            ,0           ;0x7b  '-'- keypad
.db 0            ,0           ;0x7c  '*'- keypad
.db 0            ,0           ;0x7d  '9'- keypad
.db 0            ,0           ;0x7e
.db 0            ,0           ;0x7f

